
# -*- coding: utf-8 -*-
# @File    :   __init__.py
# @Time    :   2023/11/22 15:23:19
# @Author  :   Qing 
# @Email   :   aqsz2526@outlook.com
######################### docstring ########################
'''
    声明 
        normalize_sample
        evaluate
    两个方法
'''

from .base import BaseEvalutor
from .kqa import KqaAutoEvaluator