# -*- coding: utf-8 -*-
# @File    :   __init__.py
# @Time    :   2023/11/19 15:14:13
# @Author  :   Qing 
# @Email   :   aqsz2526@outlook.com
######################### docstring ########################
'''
    用于ICL 的各种retriever

'''
""" 
BM25Retriever: 用于检索文本数据的BM25检索器
VectorRetriever: 用于检索文本数据的向量检索器
    rquirements:
        faiss-gpu
        sentence_transformers

"""